/*
 * $Id: kebookapp.cpp,v 1.10 2000/04/14 12:12:14 hige Exp $
 *
 * Copyright (C) 1999 SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
#include "kebook.h"
#include "kebookapp.h"

#include <fstream>

using namespace thread;

//
// General functions 
//
static bool cleanDir(const QDir& dir, bool remove_dir = true)
{
    QFileInfoList flist
        = *(dir.entryInfoList(QDir::Dirs | QDir::Files | QDir::Hidden));
    if (!flist.isEmpty()) {
        QFileInfo* entry = flist.first();
        while (true) {
            if (entry->fileName() !=  "." && entry->fileName() != "..") {
                if (entry->isDir()) {
                    cleanDir(QDir(entry->absFilePath()), remove_dir);
                    if (remove_dir) {
                        dir.rmdir(entry->fileName(), false);
                    }
                } else {
                    QFile::remove(entry->absFilePath());
                }
            }
            if (entry == flist.getLast())
                break;
            entry = flist.next();
        }
    }
    return true;
}
int digUniqueDir(QDir& dir, const char* tmpdir_template)
{
    QString dirname;
    do {
        QTime t = QTime::currentTime();
        dirname.sprintf(tmpdir_template, (long)t.second()*1000+t.msec());
        dir.setPath(dirname);
    } while(dir.exists());
    return mkdir(dirname, TEMP_MODE);
}
KEBookApp::KEBookApp(int& argc, char** argv, const QString& rAppName)
    : KApplication(argc, argv, rAppName),
      CEBook::CEBook(),
      m_stopped_lookingup(false),
      m_lookup_thread(EMPTY),
      m_iserror(false)
{
    m_cur_result = new LookupResult();
    // Make Font Dir in tempdir
    makeTempRouteDir();

    m_cur_error_msg = i18n("No Error.");
    
    KConfig* config = getConfig();
    
    // Choose Exactly or Fuzzy 
    config->setGroup(CFG_GENERAL_GROUP);
    m_way_to_search = (WAY_TO_SEARCH)config->readNumEntry(CFG_WAY_TO_SEARCH,
                                                    (int)EXACT);
}
KEBookApp::~KEBookApp()
{
    
}
void KEBookApp::quit()
{
    getApp()->deleteTempDir();
    KConfig* config = getConfig();
    
    config->setGroup(CFG_PATH_GROUP);
    config->writeEntry(CFG_DICT_PATH_ENTRY, getApp()->getDictPath());
    
    // Save Subbook Num
    config->setGroup(CFG_GENERAL_GROUP);
    int i = getApp()->getSubbook();
    if (i != ebook::RC_ERR) {
        config->writeEntry(CFG_SUBBOOK_PREVNUM_ENTRY, i);
        config->writeEntry(CFG_SUBBOOK_PREVDIR_ENTRY,
                           getApp()->getSubbookDirectory());
    }
    config->writeEntry(CFG_WAY_TO_SEARCH, (int)m_way_to_search);
    
    KApplication::quit();
}
void KEBookApp::getSubbookTitleList(vector<string>& list)
{
    list.erase(list.begin(), list.end());
    SubbookInfoListIterator it = getBeginOfSubbookInfoList();
    while (it != getEndOfSubbookInfoList()) {
        list.push_back((*it++).title);
    }
}
void KEBookApp::setSubbook(SubbookInfoListIterator it)
{
    try {
        CEBook::setSubbook(it);
        emit sigSetSubbook();
    } catch (ebook::CEBook_Exception& e){
        emit sigReleaseSubbook();
        return;
    }
    string fontsdir;
    if (makeTempFontsDir(fontsdir)) {
        setFontPath(fontsdir.c_str());
    }
}
void KEBookApp::setDictPath(const char* dict)
{
    if (isAvailableDictPath(dict)) {
        CEBook::setDictPath(dict);
        emit sigSetDictPath();
    }
}
void KEBookApp::lookupWord(const char* word)
{
    m_cur_result->clear();
    m_stopped_lookingup = false;
    m_iserror = false;
    try {
        CEBook::lookupWord(this, *m_cur_result, word, m_way_to_search);
    } catch (ebook::CEBook_Exception& e) {
        m_iserror = true;
        switch (e.getErrorCode()) {
        case EB_ERR_EMPTY_WORD:
            m_cur_error_msg = i18n("Input word is empty.");
            break;
        case EB_ERR_BAD_WORD:
            m_cur_error_msg = i18n("Invalid character have been used "
                                   "in your input word.");
            break;
        case EB_ERR_NO_SUCH_SEARCH:
            m_cur_error_msg
                = i18n("The currently selected dictionary don't support "
                       "the selected way to search. "
                       "Select another way.");
            break;
        default:
            m_cur_error_msg =  e.what();
            break;
        }
    }
    m_stopped_lookingup = true;
}
void KEBookApp::pushResult(const string& word, const string& content)
{
    if (!m_result_history.empty()) {
        ResultHistory::iterator i = m_cur_result_pos;
        i++;
        m_result_history.erase(i, m_result_history.end());
    }
    m_result_history.push_back(pair<string, string>(word,
                                                    content));
    m_cur_result_pos = m_result_history.end();
    m_cur_result_pos--;
}
bool KEBookApp::getCurrentResult(string& title, string& content)
{
    if (m_result_history.empty())
        return false;
    title = RESULTHISTORY_WORD(m_cur_result_pos);
    content = RESULTHISTORY_CONTENT(m_cur_result_pos);
    return true;
}
bool KEBookApp::movePrev(string& title, string& content)
{
    if (m_cur_result_pos != m_result_history.begin()) {
        m_cur_result_pos--;
    }
    title = RESULTHISTORY_WORD(m_cur_result_pos);
    content = RESULTHISTORY_CONTENT(m_cur_result_pos);
    if (m_cur_result_pos == m_result_history.begin()) {
        return false;
    } else {
        return true;
    }
}
bool KEBookApp::moveNext(string& title, string& content)
{
    ResultHistory::iterator tail = m_result_history.end();
    tail--;
    if (m_cur_result_pos != tail) {
        m_cur_result_pos++;
    }
    title = RESULTHISTORY_WORD(m_cur_result_pos);
    content = RESULTHISTORY_CONTENT(m_cur_result_pos);
    if (m_cur_result_pos == tail) {
        return false;
    } else {
        return true;
    }
}
bool KEBookApp::makeTempFontsDir(string& receive_fontsdir)
{
    QFileInfo cursubbook_temproute(m_tempdir.absPath()+QDir::separator()
        +getSubbookDirectory());
    if (cursubbook_temproute.exists()) {
        receive_fontsdir = (const char*)(cursubbook_temproute.absFilePath()
                                         +QDir::separator()+FONTS_ROUTE_DIR);
        return true;
    }
    if (mkdir(cursubbook_temproute.absFilePath(), TEMP_MODE) == ERR) {
        return false;
    }
    QString fontsdir = cursubbook_temproute.absFilePath()
        +QDir::separator()+FONTS_ROUTE_DIR;
    if (mkdir(fontsdir, TEMP_MODE) == ERR) {
        return false;
    }
    if (mkdir(fontsdir+QDir::separator()+NARROW_FONTS_DIR, TEMP_MODE) == ERR) {
        return false;
    }
    if (mkdir(fontsdir+QDir::separator()+WIDE_FONTS_DIR, TEMP_MODE) == ERR) {
        return false;
    }
    receive_fontsdir = (const char*)fontsdir;
    return true;
}
bool KEBookApp::makeTempRouteDir()
{
    QFileInfo tmproutedir_info(getenv("TMPDIR"));
    if (!tmproutedir_info.exists() || !tmproutedir_info.isDir()) {
        tmproutedir_info.setFile(DEFAULT_TEMP_PATH);
        if (!tmproutedir_info.isDir()) {
            m_tempdir.setPath("");
            cerr << "KEBook: Not found "
                 << tmproutedir_info.absFilePath()
                 << endl; 
            return false;
        }
    }
    QString tempdir = tmproutedir_info.absFilePath()
        +QDir::separator()+TEMP_TEMPLATE;

    if (digUniqueDir(m_tempdir, tempdir) == ERR) {
        return false;
    }
    return true;
}
bool KEBookApp::deleteTempDir()
{
    if (m_tempdir.exists()) {
        bool rc = cleanDir(m_tempdir);
        if (rc) {
            QString dirname = m_tempdir.dirName();
            m_tempdir.cdUp();
            m_tempdir.rmdir(dirname);
            m_tempdir.setPath("");
        }
        return rc;
    }
    return false;
}
void* KEBookApp::run(void* args)
{
    const char* word = (const char*)args;
    lookupWord(word);
    QEvent* ev = new QEvent(Event_LookupFinish);
    KEBookApp::postEvent(topWidget(), ev);
    return EMPTY;
}
void KEBookApp::start(void* args)
{
    if (m_lookup_thread == EMPTY) {
        try {
            m_lookup_thread = new Thread(this, "Look up");
        } catch (ThreadException& e) {
            cerr << e.what() << endl;
            quit();
        }
    }
    if (!m_lookup_thread->isAlive()) {
        m_lookup_thread->start(args);
    }
}
void KEBookApp::showProgress(int index, int count, int percent)
{
    KEBookProgressEvent* ev
        = new KEBookProgressEvent(Event_Progress,
                                  index, count, percent);
    postEvent(topWidget(), ev);
#ifndef HAVE_PTHREAD
    processEvents(500);
#endif
}
bool KEBookApp::rescanDictSpecListInCDROM()
{
    m_dictionary_spec_list.clear();
    vector<string> iso9660fs_list;
    ifstream is(FILE_MTAB);

    if (is) {
        while (!is.eof()) {
            string fs_spec, fs_file, fs_vfstype, fs_mntops, fs_freq, fs_passno;
            is >> fs_spec >> fs_file >> fs_vfstype
               >> fs_mntops >> fs_freq >> fs_passno;

            char ch;
            while((ch = is.get()) != '\n' && !is.eof())
                ;

            if (fs_vfstype == VFS_ISO9660) {
                iso9660fs_list.push_back(fs_file);
            }
            
        }
        is.close();
    } else {
        return false;
    }
    vector<string>::iterator it = iso9660fs_list.begin();
    CEBook cebook;
    while (it != iso9660fs_list.end()) {
        if (CEBook::isAvailableDictPath(it->c_str())) {
            cebook.setDictPath((*it).c_str());
            SubbookInfoListIterator jt = cebook.getBeginOfSubbookInfoList();
            while (jt < cebook.getEndOfSubbookInfoList()) {
                DictSpec spec;
                spec.directory = *it;
                spec.title = (*jt).title;
                spec.code = (*jt).code;
                m_dictionary_spec_list.push_back(spec);
                jt++;
            }
        }
        it++;
    }
    return true;
}
void KEBookApp::setDictionary(const DictSpec& dictspec)
{
    setDictPath(dictspec.directory.c_str());
    SubbookInfoListIterator it = getBeginOfSubbookInfoList();
    while (it < getEndOfSubbookInfoList()) {
        if (dictspec.code == (*it).code) {
            setSubbook(it);
            break;
        }
        it++;
    }
}
#include "kebookapp.moc"
/*
 * $Log: kebookapp.cpp,v $
 * Revision 1.10  2000/04/14 12:12:14  hige
 * Replace ``automoc'' with ``am_edit''.
 *
 * Revision 1.9  2000/04/07 02:03:42  hige
 * Change function from ``mainWidget'' to ``topWidget''.
 *
 * Revision 1.8  2000/04/04 12:59:21  hige
 * Change position of log in comment.
 *
 */
